import json
import sys
import webbrowser
from pathlib import Path
from typing import Annotated, Optional, cast

import typer
from pydantic import BaseModel
from rich.console import Text

from . import aws, cli
from .config import Config, Environment

app = typer.Typer()


class ContextObject(BaseModel):
    config: Config
    environment: str
    aws_profile: Optional[str] = None


@app.callback()
def common(
    ctx: typer.Context,
    config_file: Annotated[
        Optional[Path],
        typer.Option(
            "--config-file",
            "-c",
            help="Location of configuration file",
            envvar="AWS_HELPER_CONFIG_FILE",
        ),
    ] = None,
    environment: Annotated[
        str,
        typer.Option(
            "--environment",
            "-e",
            help="Environment to use from config file",
            envvar="AWS_HELPER_ENVIRONMENT",
        ),
    ] = "development",
    aws_profile: Annotated[
        Optional[str],
        typer.Option(help="AWS configuration profile to use", envvar="AWS_HELPER_AWS_PROFILE"),
    ] = None,
    quiet: Annotated[bool, typer.Option(help="Supress all non-error output")] = False,
):
    cli.set_quiet(quiet)
    ctx.obj = ContextObject(
        config=cli.load_config(config_file),
        aws_profile=aws_profile,
        environment=environment,
    )


@app.command(help="Access the AWS console")
def console(
    ctx: typer.Context,
    session_duration: Annotated[
        int, typer.Option(help="Duration of console session in seconds")
    ] = 3600,
    browser: Annotated[
        bool,
        typer.Option(
            help="Open console URL in web browser rather than writing to standard output"
        ),
    ] = True,
    tab: Annotated[
        bool,
        typer.Option(
            help="If console is opened in the browser, use a new tab rather than a new window"
        ),
    ] = True,
):
    ctx_obj = cast(ContextObject, ctx.obj)
    config = ctx_obj.config
    environment = ctx_obj.environment
    environment_config = cli.get_environment(config, environment)
    aws_creds = _aws_credentials(environment, environment_config, ctx_obj.aws_profile)

    cli.info_console.print(
        "Accessing AWS console for the",
        Text(environment, "yellow"),
        "environment.",
        style="green",
    )

    try:
        console_url = aws.get_console_url(
            aws_creds,
            issuer="aws-helper",
            region=environment_config.aws_region,
            session_duration_seconds=session_duration,
        )
    except Exception as e:
        cli.error_console.print(
            "Failed to create id AWS console URL for environment ",
            Text(environment, "yellow"),
            f". Error was: {e}",
            sep="",
        )
        sys.exit(1)

    if browser:
        if tab:
            webbrowser.open_new_tab(console_url)
        else:
            webbrowser.open_new(console_url)
    else:
        print(console_url)


@app.command(help="AWS CLI credential helper")
def cli_credentials(
    ctx: typer.Context,
):
    ctx_obj = cast(ContextObject, ctx.obj)
    config = ctx_obj.config
    environment = ctx_obj.environment
    environment_config = cli.get_environment(config, environment)
    aws_creds = _aws_credentials(environment, environment_config, ctx_obj.aws_profile)
    json.dump(
        {
            "Version": 1,
            "AccessKeyId": aws_creds.access_key_id,
            "SecretAccessKey": aws_creds.secret_access_key,
            "SessionToken": aws_creds.session_token,
            "Expiration": aws_creds.expiration.isoformat(),
        },
        sys.stdout,
    )


def _aws_credentials(
    environment: str, environment_config: Environment, profile_name: Optional[str]
) -> aws.Credentials:
    google_id_token, google_user_info = cli.google_id_token(environment_config)
    crsid = google_user_info.email.split("@")[0]
    return cli.aws_credentials(
        google_id_token,
        environment_config.aws_role_arn,
        role_session_name=f"{environment}-{crsid}",
        profile_name=profile_name,
    )


if __name__ == "__main__":
    app()
